/**
 * run simple select and update benchmark.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <libpq-fe.h>

#define TIMES 3000

static void
exit_nicely(PGconn *conn)
{
    PQfinish(conn);
    exit(1);
}

void
select_test(PGconn *conn)
{
	PGresult *res;
	for (int i = 0; i < TIMES; ++i) {
		res = PQexec(conn, "SELECT id, bar FROM foo");
		if (PQresultStatus(res) != PGRES_TUPLES_OK)
		{
			fprintf(stderr, "SELECT command failed: %s",
				PQerrorMessage(conn));
			PQclear(res);
			exit_nicely(conn);
		}
		PQclear(res);
	}
}

void
update_test(PGconn *conn)
{
	PGresult *res;
	for (int i = 0; i < TIMES; ++i) {
		res = PQexec(conn, "UPDATE foo SET bar=bar+1 WHERE id=1");
		if (PQresultStatus(res) != PGRES_COMMAND_OK)
		{
			fprintf(stderr, "UPDATE command failed: %s",
				PQerrorMessage(conn));
			PQclear(res);
			exit_nicely(conn);
		}
		PQclear(res);
	}
}

double diff_microseconds(struct timeval *start, struct timeval *end)
{
	return (double)((end->tv_sec * 1000000 + end->tv_usec) -
			(start->tv_sec * 1000000 + start->tv_usec));
}

int
main()
{
	struct timeval start, end;
	const char *conninfo = "host=localhost dbname=t1 user=t1 password=fNfwREMqO69TB9YqE+/OzF5/k+s=";
	PGconn *conn;

	conn = PQconnectdb(conninfo);
	if (PQstatus(conn) != CONNECTION_OK)
	{
		fprintf(stderr, "Connection to database failed: %s",
			PQerrorMessage(conn));
		exit_nicely(conn);
	}

	gettimeofday(&start, NULL);
	select_test(conn);
	gettimeofday(&end, NULL);
	printf("%.3f\n", diff_microseconds(&start, &end) / 1000000.0);

	gettimeofday(&start, NULL);
	update_test(conn);
	gettimeofday(&end, NULL);
	printf("%.3f\n", diff_microseconds(&start, &end) / 1000000.0);

	PQfinish(conn);
	return 0;
}
